use anyhow::Result;

use ecs::*;
use engine::prelude::*;

use crate::game::Game;

#[derive(Default, Resource)]
pub enum GameState {
    #[default]
    Startup,

    Game(Game),
}

impl GameState {
    pub fn update(
        commands: &mut Commands,
        game_state: &mut GameState,
        scene: &mut Scene,
        camera_control: &mut FreeCameraControl,
    ) -> Result<bool> {
        match game_state {
            GameState::Startup => *game_state = GameState::Game(Game),
            GameState::Game(game) => game.update(commands, scene, camera_control)?,
        }

        Ok(true)
    }
}

impl EventConsumer for GameState {
    fn event(&mut self, commands: &mut Commands, event: EngineEvent<'_>) -> Result<()> {
        match self {
            GameState::Startup => (),
            GameState::Game(game) => game.event(commands, event)?,
        }

        Ok(())
    }
}
