#[cfg(feature = "OpenXR")]
pub mod openxrintegration;

#[cfg(not(feature = "OpenXR"))]
pub mod openxrintegration {
    use crate::Result;
    use vulkan_rs::prelude::*;

    use std::sync::Arc;

    use crate::prelude::*;

    pub struct OpenXRIntegration;

    impl OpenXRIntegration {
        pub fn new(_: ApplicationInfo) -> Result<Self> {
            panic!("Feature OpenXR is not enabled!")
        }

        pub fn activate_vulkan_instance_extensions(
            &self,
            _: &mut InstanceExtensions,
        ) -> Result<()> {
            unimplemented!()
        }

        pub fn activate_vulkan_device_extensions(&self, _: &mut DeviceExtensions) -> Result<()> {
            unimplemented!()
        }

        pub fn physical_device(&self, _: &Arc<Instance>) -> Result<VkPhysicalDevice> {
            unimplemented!()
        }
    }

    impl std::fmt::Debug for OpenXRIntegration {
        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
            write!(f, "OpenXRIntegration {{ }}")
        }
    }
}

#[cfg(feature = "OpenXR")]
pub mod openxrrendercore;

#[cfg(not(feature = "OpenXR"))]
pub mod openxrrendercore {
    use crate::Result;
    use ecs::World;
    use ui::prelude::*;
    use vulkan_rs::prelude::*;

    use std::sync::{Arc, Mutex};

    use super::openxrintegration::OpenXRIntegration;

    use crate::RenderCoreCreateInfo;
    use crate::prelude::*;

    pub struct OpenXRRenderCore {
        _dummy: u32,
    }

    impl OpenXRRenderCore {
        pub fn new(
            _: &OpenXRIntegration,
            _: &Arc<Device>,
            _: &Arc<Mutex<Queue>>,
            _: RenderCoreCreateInfo,
        ) -> Result<(Self, TargetMode<()>)> {
            panic!("Feature OpenXR is not enabled!")
        }
    }

    impl OpenXRRenderCore {
        pub fn format(&self) -> VkFormat {
            unimplemented!()
        }

        pub fn resize(&mut self, _: &World, _: u32, _: u32) -> Result<()> {
            unimplemented!()
        }

        pub fn next_frame(&mut self, _: &World) -> Result<bool> {
            unimplemented!()
        }

        pub fn set_clear_color(&self, _: [f32; 4]) {
            unimplemented!()
        }

        // post process handling
        pub fn add_render_routine<T: TScene>(&mut self, _priority: u32) {
            unimplemented!()
        }

        pub fn remove_render_routine<T: TScene>(&mut self) {
            unimplemented!()
        }

        // getter
        pub fn image_count(&self) -> usize {
            unimplemented!()
        }

        pub fn images(&self) -> TargetMode<Vec<Arc<Image>>> {
            unimplemented!()
        }

        pub fn width(&self) -> u32 {
            unimplemented!()
        }

        pub fn height(&self) -> u32 {
            unimplemented!()
        }

        pub fn transformations(&self) -> Option<(VRTransformations, VRTransformations)> {
            unimplemented!()
        }
    }

    impl std::fmt::Debug for OpenXRRenderCore {
        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
            write!(f, "OpenXRRenderCore {{ }}")
        }
    }
}
